@@ -4,11 +4,24 @@ import os.path
 import ssl
 from io import BytesIO
 from pathlib import Path
-from typing import Any, AsyncIterable, Dict, Iterable, List, Optional, Type, Union
+from typing import (
+    Any,
+    AsyncIterable,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    overload,
+)
 
 import aiohttp
+from aiohttp.web import HTTPNotFound
 from aleph_message import parse_message
-from aleph_message.models import AlephMessage, ItemHash, ItemType
+from aleph_message.models import AlephMessage, ItemHash, ItemType, MessageType
+from aleph_message.status import MessageStatus
 from pydantic import ValidationError
 
 from ..conf import settings
@@ -20,13 +33,14 @@ from ..exceptions import (
 )
 from ..query.filters import MessageFilter, PostFilter
 from ..query.responses import MessagesResponse, Post, PostsResponse, PriceResponse
-from ..types import GenericMessage
+from ..types import GenericMessage, StoredContent
 from ..utils import (
     Writable,
     check_unix_socket_valid,
     copy_async_readable_to_buffer,
     extended_json_encoder,
     get_message_type_value,
+    safe_getattr,
 )
 from .abstract import AlephClient
 
@@ -343,11 +357,27 @@ class AlephHttpClient(AlephClient):
                 pagination_item=response_json["pagination_item"],
             )
 
+    @overload
     async def get_message(
         self,
         item_hash: str,
         message_type: Optional[Type[GenericMessage]] = None,
-    ) -> GenericMessage:
+    ) -> GenericMessage: ...
+
+    @overload
+    async def get_message(
+        self,
+        item_hash: str,
+        message_type: Optional[Type[GenericMessage]] = None,
+        with_status: bool = False,
+    ) -> Tuple[GenericMessage, MessageStatus]: ...
+
+    async def get_message(
+        self,
+        item_hash: str,
+        message_type: Optional[Type[GenericMessage]] = None,
+        with_status: bool = False,
+    ) -> Union[GenericMessage, Tuple[GenericMessage, MessageStatus]]:
         async with self.http_session.get(f"/api/v0/messages/{item_hash}") as resp:
             try:
                 resp.raise_for_status()
@@ -368,7 +398,10 @@ class AlephHttpClient(AlephClient):
                     f"The message type '{message.type}' "
                     f"does not match the expected type '{expected_type}'"
                 )
-        return message
+        if with_status:
+            return message, message_raw["status"]
+        else:
+            return message
 
     async def get_message_error(
         self,
@@ -428,3 +461,45 @@ class AlephHttpClient(AlephClient):
                 if e.status == 400:
                     raise InvalidHashError(f"Bad request or no such hash {item_hash}")
                 raise e
+
+    async def get_message_status(self, item_hash: str) -> MessageStatus:
+        """return Status of a message"""
+        async with self.http_session.get(f"/api/v0/messages/{item_hash}") as resp:
+            if resp.status == HTTPNotFound.status_code:
+                raise MessageNotFoundError(f"No such hash {item_hash}")
+            resp.raise_for_status()
+            result = await resp.json()
+            return MessageStatus(result["status"])
+
+    async def get_stored_content(
+        self,
+        item_hash: str,
+    ) -> StoredContent:
+        """return the underlying content for a store message"""
+
+        result, resp = None, None
+        try:
+            message: AlephMessage
+            message, status = await self.get_message(
+                item_hash=ItemHash(item_hash), with_status=True
+            )
+            if status != MessageStatus.PROCESSED:
+                resp = f"Invalid message status: {status}"
+            elif message.type != MessageType.store:
+                resp = f"Invalid message type: {message.type}"
+            elif not message.content.item_hash:
+                resp = f"Invalid CID: {message.content.item_hash}"
+            else:
+                filename = safe_getattr(message.content, "metadata.name")
+                hash = message.content.item_hash
+                url = (
+                    f"{self.api_server}/api/v0/storage/raw/"
+                    if len(hash) == 64
+                    else settings.IPFS_GATEWAY
+                ) + hash
+                result = StoredContent(filename=filename, hash=hash, url=url)
+        except MessageNotFoundError:
+            resp = f"Message not found: {item_hash}"
+        except ForgottenMessageError:
+            resp = f"Message forgotten: {item_hash}"
+        return result if result else StoredContent(error=resp)
